@@ -3,7 +3,7 @@
 #==============================================================================
 # Demonstrates how to implement a multi-entry widget for 10-digit phone numbers.
 #
-# Copyright (c) 1999-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 1999-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 package require mentry_tile
@@ -33,14 +33,16 @@ proc phoneNumberMentry {win args} {
     $win attrib type PhoneNumber
 
     #
-    # Allow only decimal digits in all entry children; use
+    # Allow only decimal digits in all entry components; use
     # wcb::cbappend (or wcb::cbprepend) instead of wcb::callback
     # in order to keep the wcb::checkEntryLen callback,
-    # registered by mentry::mentry for all entry children
+    # registered by mentry::mentry for all entry components
     #
     for {set n 0} {$n < 3} {incr n} {
+	set w [$win entrypath $n]
+	wcb::cbappend $w before insert wcb::checkStrForNum
 	$win adjustentry $n "0123456789"
-	wcb::cbappend [$win entrypath $n] before insert wcb::checkStrForNum
+	bindtags $w [linsert [bindtags $w] 1 MentryPhoneNumber]
     }
 
     return $win
@@ -81,7 +83,7 @@ proc getPhoneNumber win {
     checkIfPhoneNumberMentry $win
 
     #
-    # Generate an error if any entry child is empty or incomplete
+    # Generate an error if any entry component is empty or incomplete
     #
     for {set n 0} {$n < 3} {incr n} {
 	if {[$win isempty $n]} {
@@ -96,7 +98,7 @@ proc getPhoneNumber win {
 
     #
     # Return the phone number built from the
-    # values contained in the entry children
+    # values contained in the entry components
     #
     $win getarray strs
     return $strs(0)$strs(1)$strs(2)
@@ -119,6 +121,29 @@ proc checkIfPhoneNumberMentry win {
     }
 }
 
+bind MentryPhoneNumber <<Paste>> { pastePhoneNumber %W }
+
+#------------------------------------------------------------------------------
+# pastePhoneNumber
+#
+# Handles <<Paste>> events in the entry component w of a mentry widget for
+# 10-digit phone numbers by pasting the current contents of the clipboard into
+# the mentry if it is a valid 10-digit phone number.
+#------------------------------------------------------------------------------
+proc pastePhoneNumber w {
+    if {[llength [info procs ::tk::GetSelection]] == 1} {
+	set res [catch {::tk::GetSelection $w CLIPBOARD} num]
+    } else {					;# for Tk versions prior to 8.3
+	set res [catch {selection get -displayof $w -selection CLIPBOARD} num]
+    }
+    if {$res == 0} {
+	set win [winfo parent [winfo parent $w]]
+	catch { putPhoneNumber $num $win }
+    }
+
+    return -code break ""
+}
+
 #------------------------------------------------------------------------------
 
 #
