@@ -2,10 +2,10 @@
 # Demonstrates how to use a tablelist widget for displaying information about
 # children of an arbitrary widget.
 #
-# Copyright (c) 2000-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2000-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require tablelist_tile 6.11
+package require tablelist_tile 6.20
 
 namespace eval demo {
     variable dir [file dirname [info script]]
@@ -22,14 +22,6 @@ namespace eval demo {
 
 source [file join $demo::dir config_tile.tcl]
 
-#
-# Work around the improper appearance of the tile scrollbars in the aqua theme
-#
-if {$demo::currentTheme eq "aqua" &&
-    [package vcompare $::tk_patchLevel "8.6.10"] < 0} {
-    interp alias {} ttk::scrollbar {} ::scrollbar
-}
-
 #------------------------------------------------------------------------------
 # demo::displayChildren
 #
