@@ -4,10 +4,10 @@
 # Demonstrates how to use a tablelist widget for displaying the content of a
 # directory.
 #
-# Copyright (c) 2010-2019  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2010-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require tablelist_tile 6.8
+package require tablelist_tile 6.20
 
 #
 # Add some entries to the Tk option database
@@ -16,19 +16,12 @@ set dir [file dirname [info script]]
 source [file join $dir option_tile.tcl]
 
 #
-# Create three images
+# Create three images corresponding to the display's DPI scaling level
 #
-image create photo clsdFolderImg -file [file join $dir clsdFolder.gif]
-image create photo openFolderImg -file [file join $dir openFolder.gif]
-image create photo fileImg       -file [file join $dir file.gif]
-
-#
-# Work around the improper appearance of the tile scrollbars in the aqua theme
-#
-if {[tablelist::getCurrentTheme] eq "aqua" &&
-    [package vcompare $::tk_patchLevel "8.6.10"] < 0} {
-    interp alias {} ttk::scrollbar {} ::scrollbar
-}
+set pct $tablelist::scalingpct
+image create photo clsdFolderImg -file [file join $dir clsdFolder$pct.gif]
+image create photo openFolderImg -file [file join $dir openFolder$pct.gif]
+image create photo fileImg       -file [file join $dir       file$pct.gif]
 
 #------------------------------------------------------------------------------
 # displayContents
@@ -61,6 +54,15 @@ proc displayContents dir {
     ttk::scrollbar $vsb -orient vertical   -command [list $tbl yview]
     ttk::scrollbar $hsb -orient horizontal -command [list $tbl xview]
 
+    #
+    # On X11 configure the tablelist according
+    # to the display's DPI scaling level
+    #
+    global isAwTheme
+    if {[tk windowingsystem] eq "x11" && !$isAwTheme} {
+	$tbl configure -treestyle bicolor$tablelist::scalingpct
+    }
+
     #
     # Create a pop-up menu with one command entry; bind the script
     # associated with its entry to the <Double-1> event, too
@@ -69,6 +71,10 @@ proc displayContents dir {
     menu $menu -tearoff no
     $menu add command -label "Display Contents" \
 		      -command [list putContentsOfSelFolder $tbl]
+    if {$isAwTheme} {
+	global currentTheme
+	ttk::theme::${currentTheme}::setMenuColors $menu
+    }
     set bodyTag [$tbl bodytag]
     bind $bodyTag <<Button3>>  [bind TablelistBody <Button-1>]
     bind $bodyTag <<Button3>> +[bind TablelistBody <ButtonRelease-1>]
@@ -100,7 +106,7 @@ proc displayContents dir {
     grid $hsb -row 2 -column 0 -sticky ew
     grid rowconfigure    $tf 1 -weight 1
     grid columnconfigure $tf 0 -weight 1
-    pack $b1 $b2 $b3 -side left -expand yes -pady 10
+    pack $b1 $b2 $b3 -side left -expand yes -pady 7p
     pack $bf -side bottom -fill x
     pack $tf -side top -expand yes -fill both
 
@@ -291,7 +297,9 @@ proc expandCmd {tbl row} {
 # tablelist widget tbl.
 #------------------------------------------------------------------------------
 proc collapseCmd {tbl row} {
-    $tbl cellconfigure $row,0 -image clsdFolderImg
+    if {[$tbl hasrowattrib $row pathName]} {		;# directory item
+	$tbl cellconfigure $row,0 -image clsdFolderImg
+    }
 }
 
 #------------------------------------------------------------------------------
