@@ -1,12 +1,13 @@
-#!/usr/bin/env wish
+#! /usr/bin/env tclsh
 
 #==============================================================================
 # Demonstrates the use of embedded windows in tablelist widgets.
 #
-# Copyright (c) 2004-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2004-2023  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require tablelist_tile 6.11
+package require Tk
+package require tablelist_tile
 
 wm title . "Tile Library Scripts"
 
@@ -25,17 +26,15 @@ catch {font create TkFixedFont -family Courier -size 9}
 # Create an image corresponding to the display's DPI scaling
 # level, to be displayed in buttons embedded in a tablelist widget
 #
-set pct $tablelist::scalingpct
-image create photo openImg -file [file join $dir openAction$pct.gif]
-
-if {$currentTheme eq "aqua"} {
-    #
-    # Work around the improper appearance of the tile scrollbars
-    #
-    if {[package vcompare $::tk_patchLevel "8.6.10"] < 0} {
-	interp alias {} ttk::scrollbar {} ::scrollbar
-    }
+if {$tk_version >= 8.7 || [catch {package require tksvg}] == 0} {
+    set fmt $tablelist::svgfmt
+    image create photo viewImg -file [file join $dir view.svg] -format $fmt
 } else {
+    set pct $tablelist::scalingpct
+    image create photo viewImg -file [file join $dir view$pct.gif]
+}
+
+if {$currentTheme ne "aqua"} {
     #
     # Make the embedded buttons as small as possible.  Recall that in most
     # themes, the tile buttons consist of the following element hierarchy:
@@ -74,7 +73,8 @@ if {[$tbl cget -selectborderwidth] == 0} {
     $tbl configure -spacing 1
 }
 $tbl columnconfigure 0 -name fileName
-$tbl columnconfigure 1 -formatcommand emptyStr -sortmode integer
+$tbl columnconfigure 1 -formatcommand emptyStr -sortmode integer \
+    -stretchwindow yes
 $tbl columnconfigure 2 -name fileSize -sortmode integer
 $tbl columnconfigure 4 -name seen
 ttk::scrollbar $vsb -orient vertical -command [list $tbl yview]
@@ -114,7 +114,9 @@ if {$tk_version >= 8.5} {
     $tbl header insert 0 [list "[$tbl size] *.tcl files" "" $totalSize "" ""]
     $tbl header rowconfigure 0 -foreground blue
 
-    if {$currentTheme eq "awdark" || $currentTheme eq "black"} {
+    if {$currentTheme eq "awdark" || $currentTheme eq "awbreezedark" ||
+	$currentTheme eq "black" || $currentTheme eq "breeze-dark" ||
+	$currentTheme eq "sun-valley-dark"} {
 	$tbl header rowconfigure 0 -foreground SkyBlue
     } elseif {$currentTheme eq "aqua" &&
 	[package vcompare $tk_patchLevel "8.6.10"] >= 0} {
@@ -138,7 +140,7 @@ proc createFrame {tbl row col w} {
     # Create the frame and replace the binding tag "Frame"
     # with "TablelistBody" in the list of its binding tags
     #
-    set height [expr {[font metrics $::tblFont -linespace] * 4 / 5}]
+    set height [expr {[font metrics $::tblFont -linespace] * 9 / 10}]
     frame $w -width 72p -height $height -background ivory -borderwidth 1 \
 	     -relief solid
     bindtags $w [lreplace [bindtags $w] 1 1 TablelistBody]
@@ -165,7 +167,7 @@ proc createFrame {tbl row col w} {
 #------------------------------------------------------------------------------
 proc createButton {tbl row col w} {
     set key [$tbl getkeys $row]
-    ttk::button $w -style Embedded.TButton -image openImg -takefocus 0 \
+    ttk::button $w -style Embedded.TButton -image viewImg -takefocus 0 \
 		   -command [list viewFile $tbl $key]
 }
 
@@ -195,6 +197,7 @@ proc viewFile {tbl key} {
     set txt $tf.txt
     set vsb $tf.vsb
     text $txt -font TkFixedFont -setgrid yes -yscrollcommand [list $vsb set]
+    catch {$txt configure -tabstyle wordprocessor}
     global isAwTheme
     if {$isAwTheme} {
 	global currentTheme
@@ -236,11 +239,8 @@ proc viewFile {tbl key} {
 #
 # Create embedded windows in the columns no. 1 and 3
 #
-set rowCount [$tbl size]
-for {set row 0} {$row < $rowCount} {incr row} {
-    $tbl cellconfigure $row,1 -window createFrame -stretchwindow yes
-    $tbl cellconfigure $row,3 -window createButton
-}
+$tbl fillcolumn 1 -window createFrame
+$tbl fillcolumn 3 -window createButton
 
 set bf .bf
 ttk::frame $bf
