@@ -1,14 +1,15 @@
-#!/usr/bin/env wish
+#! /usr/bin/env tclsh
 
 #==============================================================================
 # Demonstrates the interactive tablelist cell editing with the aid of Bryan
 # Oakley's combobox, the mentry widgets of type "Date" and "Time", and of the
 # Tk core entry, spinbox, checkbutton, and menubutton widgets.
 #
-# Copyright (c) 2004-2019  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2004-2023  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require tablelist_tile 6.8
+package require Tk
+package require tablelist_tile
 package require combobox
 package require mentry
 
@@ -26,6 +27,12 @@ option add *Tablelist*Combobox.background		white
 option add *Tablelist*Combobox.elementBorderWidth	2
 option add *Tablelist*Mentry.background			white
 
+#
+# Create the images "checkedImg" and "uncheckedImg", as well as 16 images of
+# names like "img#FF0000", displaying colors identified by names like "red"
+#
+source [file join $dir images.tcl]
+
 #
 # Register Bryan Oakley's combobox widget as well as the mentry
 # widgets of type "Date" and "Time" for interactive cell editing
@@ -34,12 +41,6 @@ tablelist::addOakleyCombobox
 tablelist::addDateMentry Ymd -
 tablelist::addTimeMentry HMS :
 
-#
-# Create the images "checkedImg" and "uncheckedImg", as well as 16 images of
-# names like "img#FF0000", displaying colors identified by names like "red"
-#
-source [file join $dir images.tcl]
-
 #
 # Improve the window's appearance by using a tile
 # frame as a container for the other widgets
@@ -69,7 +70,7 @@ if {[$tbl cget -selectborderwidth] == 0} {
 }
 $tbl columnconfigure 0 -sortmode integer
 $tbl columnconfigure 1 -name available -editable yes -editwindow checkbutton \
-    -formatcommand emptyStr
+    -formatcommand emptyStr -labelwindow checkbutton
 $tbl columnconfigure 2 -name lineName  -editable yes -editwindow entry \
     -sortmode dictionary
 $tbl columnconfigure 3 -name baudRate  -editable yes -editwindow combobox \
@@ -90,25 +91,17 @@ proc formatDate val { return [clock format $val -format "%Y-%m-%d"] }
 proc formatTime val { return [clock format $val -format "%H:%M:%S"] }
 
 #
-# Populate the tablelist widget; set the activation
-# date & time to 10 minutes past the current clock value
+# Populate the tablelist widget and configure the checkbutton
+# embedded into the header label of the column "available"
 #
-set clock [expr {[clock seconds] + 600}]
-for {set i 0; set n 1} {$i < 16} {set i $n; incr n} {
-    $tbl insert end [list $n [expr {$i < 8}] "Line $n" 9600 8 None 1 XON/XOFF \
-	$clock $clock [lindex $colorNames $i]]
-
-    set availImg [expr {($i < 8) ? "checkedImg" : "uncheckedImg"}]
-    $tbl cellconfigure end,available -image $availImg
-    $tbl cellconfigure end,color -image img[lindex $colorValues $i]
-}
+source [file join $dir serialParams.tcl]
 
 set btn [ttk::button $f.btn -text "Close" -command exit]
 
 #
 # Manage the widgets
 #
-pack $btn -side bottom -pady 10
+pack $btn -side bottom -pady 7p
 pack $tbl -side top -expand yes -fill both
 pack $f -expand yes -fill both
 
@@ -218,10 +211,12 @@ proc editEndCmd {tbl row col text} {
     switch [$tbl columncget $col -name] {
 	available {
 	    #
-	    # Update the image contained in the cell
+	    # Update the image contained in the cell and the checkbutton
+	    # embedded into the header label of the column "available"
 	    #
 	    set img [expr {$text ? "checkedImg" : "uncheckedImg"}]
 	    $tbl cellconfigure $row,$col -image $img
+	    after idle [list updateCkbtn $tbl $row $col]
 	}
 
 	baudRate {
@@ -240,7 +235,7 @@ proc editEndCmd {tbl row col text} {
 	    #
 	    # Check whether the last argument is a clock value in seconds
 	    #
-	    if {![string is digit $text]} {
+	    if {[info exists ::msgs($text)]} {
 		bell
 		tk_messageBox -title "Error" -icon error -message $::msgs($text)
 		$tbl rejectinput
@@ -269,7 +264,7 @@ proc editEndCmd {tbl row col text} {
 	    #
 	    # Check whether the last argument is a clock value in seconds
 	    #
-	    if {![string is digit $text]} {
+	    if {[info exists ::msgs($text)]} {
 		bell
 		tk_messageBox -title "Error" -icon error -message $::msgs($text)
 		$tbl rejectinput
