@@ -1,13 +1,23 @@
-#!/usr/bin/env wish
+#! /usr/bin/env tclsh
 
 #==============================================================================
 # Demonstrates the interactive tablelist cell editing with the aid of some
 # widgets from the tile package.
 #
-# Copyright (c) 2005-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2005-2023  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require tablelist_tile 6.11
+package require Tk
+package require tablelist_tile
+
+if {[tk windowingsystem] eq "x11" &&
+    ($tk_version < 8.7 || [package vcompare $::tk_patchLevel "8.7a5"] <= 0)} {
+    #
+    # Patch the default theme's styles TCheckbutton and TRadiobutton
+    #
+    package require themepatch
+    themepatch::patch default
+}
 
 wm title . "Serial Line Configuration"
 
@@ -53,7 +63,7 @@ tablelist::tablelist $tbl \
 	      0 "Cable Color"	  center} \
     -editstartcommand editStartCmd -editendcommand editEndCmd \
     -height 0 -width 0
-if {$isAwTheme && ![regexp {^(aw)?(arc|breeze)$} $currentTheme]} {
+if {$isAwTheme && ![regexp {^(aw)?(arc|breeze.*)$} $currentTheme]} {
     $tbl configure -borderwidth 2
 }
 if {[$tbl cget -selectborderwidth] == 0} {
@@ -61,7 +71,8 @@ if {[$tbl cget -selectborderwidth] == 0} {
 }
 $tbl columnconfigure 0 -sortmode integer
 $tbl columnconfigure 1 -name available -editable yes \
-    -editwindow ttk::checkbutton -formatcommand emptyStr
+    -editwindow ttk::checkbutton -formatcommand emptyStr \
+    -labelwindow ttk::checkbutton
 $tbl columnconfigure 2 -name lineName  -editable yes -editwindow ttk::entry \
     -sortmode dictionary
 $tbl columnconfigure 3 -name baudRate  -editable yes -editwindow ttk::combobox \
@@ -86,18 +97,10 @@ proc formatDate val { return [clock format $val -format "%Y-%m-%d"] }
 proc formatTime val { return [clock format $val -format "%H:%M:%S"] }
 
 #
-# Populate the tablelist widget; set the activation
-# date & time to 10 minutes past the current clock value
+# Populate the tablelist widget and configure the checkbutton
+# embedded into the header label of the column "available"
 #
-set clock [expr {[clock seconds] + 600}]
-for {set i 0; set n 1} {$i < 16} {set i $n; incr n} {
-    $tbl insert end [list $n [expr {$i < 8}] "Line $n" 9600 8 None 1 XON/XOFF \
-	$clock $clock [lindex $colorNames $i]]
-
-    set availImg [expr {($i < 8) ? "checkedImg" : "uncheckedImg"}]
-    $tbl cellconfigure end,available -image $availImg
-    $tbl cellconfigure end,color -image img[lindex $colorValues $i]
-}
+source [file join $dir serialParams.tcl]
 
 if {$::argc == 0} {
     set btn [ttk::button $f.btn -text "Close" -command exit]
@@ -219,10 +222,12 @@ proc editEndCmd {tbl row col text} {
     switch [$tbl columncget $col -name] {
 	available {
 	    #
-	    # Update the image contained in the cell
+	    # Update the image contained in the cell and the checkbutton
+	    # embedded into the header label of the column "available"
 	    #
 	    set img [expr {$text ? "checkedImg" : "uncheckedImg"}]
 	    $tbl cellconfigure $row,$col -image $img
+	    after idle [list updateCkbtn $tbl $row $col]
 	}
 
 	baudRate {
@@ -239,7 +244,7 @@ proc editEndCmd {tbl row col text} {
 
 	actDate {
 	    #
-	    # Get the activation date in seconds from the last argument 
+	    # Get the activation date in seconds from the last argument
 	    #
 	    if {[catch {clock scan $text} actDate] != 0} {
 		bell
@@ -268,7 +273,7 @@ proc editEndCmd {tbl row col text} {
 
 	actTime {
 	    #
-	    # Get the activation clock value in seconds from the last argument 
+	    # Get the activation clock value in seconds from the last argument
 	    #
 	    set actDate [$tbl cellcget $row,actDate -text]
 	    if {[catch {clock scan $text -base $actDate} actClock] != 0} {
