@@ -11,13 +11,12 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# RCS: @(#) $Id: chatwidget.tcl,v 1.4 2008/06/20 22:53:54 patthoyts Exp $
 
 package require Tcl 8.5
 package require Tk 8.5
 
 namespace eval chatwidget {
-    variable version 1.1.0
+    variable version 1.1.4
 
     namespace export chatwidget
 
@@ -32,6 +31,9 @@ namespace eval chatwidget {
             [font configure ChatwidgetFont] -weight bold
         eval [list font create ChatwidgetItalicFont] \
             [font configure ChatwidgetFont] -slant italic
+        eval [list font create ChatwidgetTopicFont] \
+            [font configure ChatwidgetFont] \
+            -size [expr {[font configure ChatwidgetFont -size] - 0}]
     }
 }
 
@@ -79,7 +81,7 @@ proc chatwidget::WidgetProc {self cmd args} {
         peer {
             return [uplevel 1 [list [namespace origin Peer] $self] $args]
         }
-        chat - 
+        chat -
         default {
             return [uplevel 1 [list [namespace origin Chat] $self] $args]
         }
@@ -146,7 +148,14 @@ proc chatwidget::Topic {self cmd args} {
     switch -exact -- $cmd {
         show { grid $self.topic -row 0 -column 0 -sticky new }
         hide { grid forget $self.topic }
-        set  { set state(topic) [lindex $args 0] }
+        set  {
+            set state(topic) [lindex $args 0]
+            $self.topic.text configure -state normal
+            $self.topic.text delete 1.0 end
+            $self.topic.text insert end $state(topic)
+            $self.topic.text configure -state disabled
+            bind $self.topic.text <Map> [list [namespace origin TopicUpdate] $self]
+        }
         default {
             return -code error "bad option \"$cmd\":\
                 must be show, hide or set"
@@ -154,6 +163,22 @@ proc chatwidget::Topic {self cmd args} {
     }
 }
 
+# Set the topic widget to 2 lines with an optional scrollbar if the text
+# will require more than a single line of display.
+proc chatwidget::TopicUpdate {self} {
+    bind $self.topic.text <Map> {}
+    set lines [$self.topic.text count -displaylines 1.0 end]
+    if {$lines < 2} {
+        $self.topic.text configure -height 1
+    } else {
+        $self.topic.text configure -height 2
+        ttk::scrollbar $self.topic.vs -command [list $self.topic.text yview]
+        $self.topic.text configure -yscrollcommand \
+            [list [namespace origin scroll_set] $self.topic.vs $self 0]
+        grid $self.topic.vs -row 0 -column 2 -sticky new -pady {2 0} -padx 1
+    }
+}
+
 proc chatwidget::Names {self args} {
     upvar #0 [namespace current]::$self state
     set frame [winfo parent $state(names_widget)]
@@ -162,12 +187,18 @@ proc chatwidget::Names {self args} {
         return $state(names_widget)
     }
     if {[llength $args] == 1 && [lindex $args 0] eq "hide"} {
-        return [$pane forget $frame]
+        if {$frame in [$pane panes]} {
+            $pane forget $frame
+        }
+        return
     }
     if {[llength $args] == 1 && [lindex $args 0] eq "show"} {
-        return [$pane add $frame]
+        if {$frame ni [$pane panes]} {
+            $pane add $frame
+        }
+        return
     }
-    return [uplevel 1 [list $state(names_widget)] $args] 
+    return [uplevel 1 [list $state(names_widget)] $args]
 }
 
 proc chatwidget::Entry {self args} {
@@ -266,6 +297,7 @@ proc chatwidget::Name {self cmd args} {
                     return $state(names)
                 }
                 default {
+                    set r {}
                     foreach item $state(names) { lappend r [lindex $item 0] }
                     return $r
                 }
@@ -278,7 +310,8 @@ proc chatwidget::Name {self cmd args} {
             }
             set nick [lindex $args 0]
             if {[set ndx [lsearch -exact -index 0 $state(names) $nick]] == -1} {
-                array set opts {-group {} -colour black}
+                set fg [$state(chat_widget) cget -foreground]
+                array set opts [list -group {} -color $fg]
                 array set opts [lrange $args 1 end]
                 lappend state(names) [linsert [array get opts] 0 $nick]
             } else {
@@ -337,7 +370,7 @@ proc chatwidget::UpdateNamesExec {self} {
     unset state(updatenames)
     set names $state(names_widget)
     set chat  $state(chat_widget)
-    
+
     foreach tagname [lsearch -all -inline [$names tag names] NICK-*] {
         $names tag delete $tagname
     }
@@ -468,7 +501,8 @@ proc chatwidget::Create {self} {
     # Create a topic/subject header
     set topic [ttk::frame $self.topic]
     ttk::label $topic.label -anchor w -text Topic
-    ttk::entry $topic.text -state disabled -textvariable [set State](topic)
+    text $topic.text -state disabled -font ChatwidgetTopicFont
+    $topic.text configure -state disabled -height 1 -wrap word
     grid $topic.label $topic.text -sticky new -pady {2 0} -padx 1
     Grid $topic 0 1
 
@@ -510,10 +544,12 @@ proc chatwidget::Create {self} {
     bindtags $chat [linsert [bindtags $chat] 1 ChatwidgetText]
     set state(chat_widget) $chat
     set state(chat_peer_widget) $peer
-    
+
     # Create the entry widget
     set entry [ttk::frame $outer.entry -style ChatwidgetFrame]
     text $entry.text -borderwidth 0 -relief flat -font ChatwidgetFont
+    $entry.text configure -insertbackground [$entry.text cget -foreground]
+
     ttk::scrollbar $entry.vs -command [list $entry.text yview]
     $entry.text configure -height 1 \
         -yscrollcommand [list [namespace origin scroll_set] $entry.vs $outer 0]
@@ -546,8 +582,10 @@ proc chatwidget::Create {self} {
         }
     }
 
-    $names.text tag configure SUBTITLE \
-        -background grey80 -font ChatwidgetBoldFont
+    # Use inverted colors for the subtitles.
+    $names.text tag configure SUBTITLE -font ChatwidgetBoldFont \
+        -foreground [$names.text cget -background] \
+        -background [$names.text cget -foreground]
     $chat tag configure NICK        -font ChatwidgetBoldFont
     $chat tag configure TYPE-system -font ChatwidgetItalicFont
     $chat tag configure URL         -underline 1
@@ -556,7 +594,7 @@ proc chatwidget::Create {self} {
     $inner add $names
     $outer add $inner -weight 1
     $outer add $entry
-    
+
     grid $outer -row 1 -column 0 -sticky news -padx 1 -pady 1
     Grid $self 1 0
     return $self
@@ -568,14 +606,14 @@ proc chatwidget::Self {widget} {
         set w [winfo parent $w]
     }
     if {![winfo exists $w]} {
-        return -code error "invalid window $widget" 
+        return -code error "invalid window $widget"
     }
     return $w
 }
 
 # Set initial position of sash
 proc chatwidget::PaneMap {w pane offset} {
-    bind $pane <Map> {}
+    bind $w <Map> {}
     if {[llength [$pane panes]] > 1} {
         if {$offset < 0} {
             if {[$pane cget -orient] eq "horizontal"} {
@@ -619,7 +657,7 @@ proc chatwidget::scroll_set {scrollbar pw set f1 f2} {
             } else {
                 grid $scrollbar
             }
-        
+
     }
     if {$set} {
         upvar #0 [namespace current]::[Self $scrollbar] state
@@ -707,13 +745,14 @@ proc chatwidget::NickcompleteCleanup {self} {
 # Update the widget chatstate (one of active, composing, paused, inactive, gone)
 # These are from XEP-0085 but seem likey useful in many chat-type environments.
 # Note: this state is _per-widget_. This is not the same as [tk inactive]
-# active = got focus and recently active
+#   active = got focus and recently active
 #   composing = typing
 #   paused = 5 secs non typing
-# inactive = no activity for 30 seconds
-# gone = no activity for 2 minutes or closed the window
+#   inactive = no activity for 30 seconds
+#   gone = no activity for 2 minutes or closed the window
 proc chatwidget::Chatstate {self what} {
     upvar #0 [namespace current]::$self state
+    if {![info exists state]} { return }
     after cancel $state(chatstatetimer)
     switch -exact -- $what {
         composing - active {
@@ -734,5 +773,5 @@ proc chatwidget::Chatstate {self what} {
         event generate $self <<ChatwidgetChatstate>>
     }
 }
-    
+
 package provide chatwidget $chatwidget::version
