@@ -1,7 +1,7 @@
 #==============================================================================
 # Contains the implementation of a multi-entry widget for IP addresses.
 #
-# Copyright (c) 1999-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 1999-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -17,9 +17,20 @@ namespace eval mentry {
     bind MentryIPAddr <Down>	{ mentry::incrIPAddrComp %W -1 }
     bind MentryIPAddr <Prior>	{ mentry::incrIPAddrComp %W  10 }
     bind MentryIPAddr <Next>	{ mentry::incrIPAddrComp %W -10 }
+    bind MentryIPAddr <<Paste>>	{ mentry::pasteIPAddr %W }
     variable winSys
-    if {[string compare $winSys "classic"] == 0 ||
-	[string compare $winSys "aqua"] == 0} {
+    variable uniformWheelSupport
+    if {$uniformWheelSupport} {
+	bind MentryIPAddr <MouseWheel> {
+	    mentry::incrIPAddrComp %W \
+		[expr {%D > 0 ? (%D + 119) / 120 : %D / 120}]
+	}
+	bind MentryIPAddr <Option-MouseWheel> {
+	    mentry::incrIPAddrComp %W \
+		[expr {%D > 0 ? (%D + 11) / 12 : %D / 12}]
+	}
+    } elseif {[string compare $winSys "classic"] == 0 ||
+	      [string compare $winSys "aqua"] == 0} {
 	catch {
 	    bind MentryIPAddr <MouseWheel> {
 		mentry::incrIPAddrComp %W %D
@@ -72,14 +83,14 @@ proc mentry::ipAddrMentry {win args} {
     ::$win attrib type IPAddr
 
     #
-    # In each entry child allow only unsigned integers of max.
+    # In each entry component allow only unsigned integers of max.
     # value 255, and insert the binding tag MentryIPAddr in the
     # list of binding tags of the entry, just after its path name
     #
     for {set n 0} {$n < 4} {incr n} {
-	::$win adjustentry $n "0123456789"
 	set w [::$win entrypath $n]
 	wcb::cbappend $w before insert "wcb::checkEntryForUInt 255"
+	::$win adjustentry $n "0123456789"
 	bindtags $w [linsert [bindtags $w] 1 MentryIPAddr]
     }
 
@@ -126,7 +137,7 @@ proc mentry::getIPAddr win {
     checkIfIPAddrMentry $win
 
     #
-    # Scan the contents of the entry children;
+    # Scan the contents of the entry components;
     # generate an error if any of them is empty
     #
     for {set n 0} {$n < 4} {incr n} {
@@ -168,8 +179,8 @@ proc mentry::checkIfIPAddrMentry win {
 # mentry::incrIPAddrComp
 #
 # This procedure handles <Up>, <Down>, <Prior>, and <Next> events in the entry
-# child w of a mentry widget for IP addresses.  It increments the entry's value
-# by the specified amount if allowed.
+# component w of a mentry widget for IP addresses.  It increments the entry's
+# value by the specified amount if allowed.
 #------------------------------------------------------------------------------
 proc mentry::incrIPAddrComp {w amount} {
     set str [$w get]
@@ -210,3 +221,24 @@ proc mentry::incrIPAddrComp {w amount} {
 	_$w icursor $oldPos
     }
 }
+
+#------------------------------------------------------------------------------
+# mentry::pasteIPAddr
+#
+# This procedure handles <<Paste>> events in the entry component w of a mentry
+# widget for IP addresses by pasting the current contents of the clipboard into
+# the mentry if it is a valid IP address.
+#------------------------------------------------------------------------------
+proc mentry::pasteIPAddr w {
+    if {[llength [info procs ::tk::GetSelection]] == 1} {
+	set res [catch {::tk::GetSelection $w CLIPBOARD} addr]
+    } else {					;# for Tk versions prior to 8.3
+	set res [catch {selection get -displayof $w -selection CLIPBOARD} addr]
+    }
+    if {$res == 0} {
+	parseChildPath $w win n
+	catch { putIPAddr $addr $win }
+    }
+
+    return -code break ""
+}
