@@ -3,27 +3,18 @@
 # values of some mentry configuration options.
 #
 # Structure of the module:
-#   - Public procedures related to tile themes
+#   - Public procedure related to tile themes
 #   - Private procedures related to tile themes
 #   - Private procedures related to global KDE configuration options
 #
-# Copyright (c) 2006-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2006-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
-# Public procedures related to tile themes
-# ========================================
+# Public procedure related to tile themes
+# =======================================
 #
 
-#------------------------------------------------------------------------------
-# mentry::getCurrentTheme
-#
-# Returns the current tile theme.
-#------------------------------------------------------------------------------
-proc mentry::getCurrentTheme {} {
-    return [mwutil::currentTheme]
-}
-
 #------------------------------------------------------------------------------
 # mentry::setThemeDefaults
 #
@@ -31,16 +22,29 @@ proc mentry::getCurrentTheme {} {
 # mentry configuration options and updates the array configSpecs.
 #------------------------------------------------------------------------------
 proc mentry::setThemeDefaults {} {
-    set currentTheme [mwutil::currentTheme]
-    variable isAwTheme \
-	[llength [info commands ::ttk::theme::${currentTheme}::setTextColors]]
-
+    #
+    # For several themes, some of the following most frequent
+    # values will be overridden by theme-specific ones:
+    #
     variable themeDefaults
+    array set themeDefaults [list \
+	-background		white \
+	-disabledbackground	"" \
+	-foreground		black \
+	-foreground,background	black \
+	-font			TkTextFont \
+    ]
+
     if {[info exists themeDefaults(-readonlybackground)]} {
 	unset themeDefaults(-readonlybackground)
     }
 
-    if {[catch {${currentTheme}Theme}] != 0} {
+    set currentTheme [::mwutil::currentTheme]
+    variable isAwTheme \
+	[llength [info commands ::ttk::theme::${currentTheme}::setTextColors]]
+    if {$isAwTheme} {
+	awTheme $currentTheme
+    } elseif {[catch {${currentTheme}Theme}] != 0} {
 	#
 	# Fall back to the "default" theme (which is the root of all
 	# themes) and then override the options set by the current one
@@ -98,7 +102,6 @@ proc mentry::setThemeDefaults {} {
 	set themeDefaults(-readonlybackground) \
 	    $themeDefaults(-disabledbackground)
     }
-    set themeDefaults(-font) TkTextFont
 
     variable configSpecs
     foreach opt {-background -foreground -font} {
@@ -115,16 +118,45 @@ proc mentry::setThemeDefaults {} {
 # =========================================
 #
 
+#------------------------------------------------------------------------------
+# mentry::awTheme
+#------------------------------------------------------------------------------
+proc mentry::awTheme theme {
+    switch $theme {
+	awarc - arc -
+	awbreeze - breeze -
+	awbreezedark			{ set bdWidth 3; set labelPadY {3 3} }
+	awblack - black -
+	awclearlooks - clearlooks -
+	awdark -
+	awlight -
+	awtemplate -
+	awwinxpblue - winxpblue -
+	default				{ set bdWidth 2; set labelPadY {2 2} }
+    }
+
+    variable themeDefaults
+    array set themeDefaults [list \
+	-background		[styleConfig . -fieldbackground] \
+	-disabledbackground	[lindex [style map TEntry -fieldbackground] 1] \
+	-foreground		[styleConfig TEntry -foreground] \
+	-foreground,background	[styleConfig TEntry -foreground] \
+	-disabledforeground	[lindex [style map TEntry -foreground] 1] \
+	-selectbackground	[styleConfig . -selectbackground] \
+	-selectforeground	[styleConfig . -selectforeground] \
+	-selectborderwidth	[styleConfig . -selectborderwidth] \
+	-borderwidth		$bdWidth \
+	-labelpady		$labelPadY \
+    ]
+}
+
 #------------------------------------------------------------------------------
 # mentry::altTheme
 #------------------------------------------------------------------------------
 proc mentry::altTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
 	-disabledbackground	#d9d9d9 \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#a3a3a3 \
 	-selectbackground	#4a6984 \
 	-selectforeground	#ffffff \
@@ -157,7 +189,6 @@ proc mentry::aquaTheme {} {
 
 	array set themeDefaults [list \
 	    -background			$background \
-	    -disabledbackground		"" \
 	    -foreground			systemTextColor \
 	    -foreground,background	systemTextColor \
 	    -disabledforeground		systemDisabledControlTextColor \
@@ -170,7 +201,6 @@ proc mentry::aquaTheme {} {
     } else {
 	array set themeDefaults [list \
 	    -background			systemWindowBody \
-	    -disabledbackground		"" \
 	    -foreground			systemModelessDialogActiveText \
 	    -foreground,background	systemModelessDialogInactiveText \
 	    -disabledforeground		systemModelessDialogInactiveText \
@@ -189,10 +219,6 @@ proc mentry::aquaTheme {} {
 proc mentry::AquativoTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	black \
 	-selectbackground	#000000 \
 	-selectforeground	#ffffff \
@@ -208,10 +234,6 @@ proc mentry::AquativoTheme {} {
 proc mentry::aquativoTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#565248 \
 	-selectbackground	#000000 \
 	-selectforeground	#ffffff \
@@ -227,7 +249,6 @@ proc mentry::aquativoTheme {} {
 proc mentry::ArcTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
 	-disabledbackground	#fbfcfc \
 	-foreground		#5c616c \
 	-foreground,background	#5c616c \
@@ -240,114 +261,6 @@ proc mentry::ArcTheme {} {
     ]
 }
 
-#------------------------------------------------------------------------------
-# mentry::arcTheme
-#------------------------------------------------------------------------------
-proc mentry::arcTheme {} {
-    variable isAwTheme
-    if {$isAwTheme} {
-	awarcTheme 
-    } else {
-	return -code error			;# handled by setThemeDefaults
-    }
-}
-
-#------------------------------------------------------------------------------
-# mentry::awarcTheme
-#------------------------------------------------------------------------------
-proc mentry::awarcTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	3 \
-	-labelpady	{3 3} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::awblackTheme
-#------------------------------------------------------------------------------
-proc mentry::awblackTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	2 \
-	-labelpady	{2 2} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::awbreezeTheme
-#------------------------------------------------------------------------------
-proc mentry::awbreezeTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	3 \
-	-labelpady	{3 3} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::awclearlooksTheme
-#------------------------------------------------------------------------------
-proc mentry::awclearlooksTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	2 \
-	-labelpady	{2 2} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::awdarkTheme
-#------------------------------------------------------------------------------
-proc mentry::awdarkTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	2 \
-	-labelpady	{2 2} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::awlightTheme
-#------------------------------------------------------------------------------
-proc mentry::awlightTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	2 \
-	-labelpady	{2 2} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::awwinxpblueTheme
-#------------------------------------------------------------------------------
-proc mentry::awwinxpblueTheme {} {
-    awTheme 
-    variable themeDefaults
-    array set themeDefaults [list \
-	-borderwidth	2 \
-	-labelpady	{2 2} \
-    ]
-}
-
-#------------------------------------------------------------------------------
-# mentry::blackTheme
-#------------------------------------------------------------------------------
-proc mentry::blackTheme {} {
-    variable isAwTheme
-    if {$isAwTheme} {
-	awblackTheme 
-    } else {
-	return -code error			;# handled by setThemeDefaults
-    }
-}
-
 #------------------------------------------------------------------------------
 # mentry::blueTheme
 #------------------------------------------------------------------------------
@@ -355,9 +268,6 @@ proc mentry::blueTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
 	-background		#e6f3ff \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#666666 \
 	-selectbackground	#ffff33 \
 	-selectforeground	#000000 \
@@ -367,28 +277,12 @@ proc mentry::blueTheme {} {
     ]
 }
 
-#------------------------------------------------------------------------------
-# mentry::breezeTheme
-#------------------------------------------------------------------------------
-proc mentry::breezeTheme {} {
-    variable isAwTheme
-    if {$isAwTheme} {
-	awbreezeTheme 
-    } else {
-	return -code error			;# handled by setThemeDefaults
-    }
-}
-
 #------------------------------------------------------------------------------
 # mentry::clamTheme
 #------------------------------------------------------------------------------
 proc mentry::clamTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#999999 \
 	-selectbackground	#4a6984 \
 	-selectforeground	#ffffff \
@@ -404,10 +298,7 @@ proc mentry::clamTheme {} {
 proc mentry::classicTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
 	-disabledbackground	#d9d9d9 \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#a3a3a3 \
 	-selectbackground	#c3c3c3 \
 	-selectforeground	#000000 \
@@ -421,25 +312,16 @@ proc mentry::classicTheme {} {
 # mentry::clearlooksTheme
 #------------------------------------------------------------------------------
 proc mentry::clearlooksTheme {} {
-    variable isAwTheme
-    if {$isAwTheme} {
-	awclearlooksTheme 
-    } else {
-	variable themeDefaults
-	array set themeDefaults [list \
-	    -background			white \
-	    -disabledbackground		"" \
-	    -readonlybackground		#efebe7 \
-	    -foreground			black \
-	    -foreground,background	black \
-	    -disabledforeground		#b5b3ac \
-	    -selectbackground		#7c99ad \
-	    -selectforeground		#ffffff \
-	    -selectborderwidth		0 \
-	    -borderwidth		2 \
-	    -labelpady			{2 2} \
-	]
-    }
+    variable themeDefaults
+    array set themeDefaults [list \
+	-readonlybackground	#efebe7 \
+	-disabledforeground	#b5b3ac \
+	-selectbackground	#7c99ad \
+	-selectforeground	#ffffff \
+	-selectborderwidth	0 \
+	-borderwidth		2 \
+	-labelpady		{2 2} \
+    ]
 }
 
 #------------------------------------------------------------------------------
@@ -448,10 +330,7 @@ proc mentry::clearlooksTheme {} {
 proc mentry::defaultTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
 	-disabledbackground	#d9d9d9 \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#a3a3a3 \
 	-selectbackground	#4a6984 \
 	-selectforeground	#ffffff \
@@ -467,10 +346,6 @@ proc mentry::defaultTheme {} {
 proc mentry::keramikTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#aaaaaa \
 	-selectbackground	#0a5f89 \
 	-selectforeground	#ffffff \
@@ -486,10 +361,6 @@ proc mentry::keramikTheme {} {
 proc mentry::keramik_altTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#aaaaaa \
 	-selectbackground	#0a5f89 \
 	-selectforeground	#ffffff \
@@ -505,10 +376,6 @@ proc mentry::keramik_altTheme {} {
 proc mentry::krocTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#b2b2b2 \
 	-selectbackground	#000000 \
 	-selectforeground	#ffffff \
@@ -524,10 +391,6 @@ proc mentry::krocTheme {} {
 proc mentry::plastikTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#aaaaaa \
 	-selectbackground	#657a9e \
 	-selectforeground	#ffffff \
@@ -544,9 +407,6 @@ proc mentry::srivTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
 	-background		#e6f3ff \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#666666 \
 	-selectbackground	#ffff33 \
 	-selectforeground	#000000 \
@@ -563,9 +423,6 @@ proc mentry::srivlgTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
 	-background		#e6f3ff \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#666666 \
 	-selectbackground	#ffff33 \
 	-selectforeground	#000000 \
@@ -581,10 +438,6 @@ proc mentry::srivlgTheme {} {
 proc mentry::stepTheme {} {
     variable themeDefaults
     array set themeDefaults [list \
-	-background		white \
-	-disabledbackground	"" \
-	-foreground		black \
-	-foreground,background	black \
 	-disabledforeground	#808080 \
 	-selectbackground	#fdcd00 \
 	-selectforeground	#ffffff \
@@ -662,24 +515,15 @@ proc mentry::winnativeTheme {} {
 # mentry::winxpblueTheme
 #------------------------------------------------------------------------------
 proc mentry::winxpblueTheme {} {
-    variable isAwTheme
-    if {$isAwTheme} {
-	awwinxpblueTheme 
-    } else {
-	variable themeDefaults
-	array set themeDefaults [list \
-	    -background			white \
-	    -disabledbackground		"" \
-	    -foreground			black \
-	    -foreground,background	black \
-	    -disabledforeground		#565248 \
-	    -selectbackground		#4a6984 \
-	    -selectforeground		#ffffff \
-	    -selectborderwidth		0 \
-	    -borderwidth		2 \
-	    -labelpady			{2 2} \
-	]
-    }
+    variable themeDefaults
+    array set themeDefaults [list \
+	-disabledforeground	#565248 \
+	-selectbackground	#4a6984 \
+	-selectforeground	#ffffff \
+	-selectborderwidth	0 \
+	-borderwidth		2 \
+	-labelpady		{2 2} \
+    ]
 }
 
 #------------------------------------------------------------------------------
@@ -701,23 +545,6 @@ proc mentry::xpnativeTheme {} {
     ]
 }
 
-#------------------------------------------------------------------------------
-# mentry::awTheme
-#------------------------------------------------------------------------------
-proc mentry::awTheme {} {
-    variable themeDefaults
-    array set themeDefaults [list \
-	-background		[styleConfig . -fieldbackground] \
-	-disabledbackground	[lindex [style map TEntry -fieldbackground] 1] \
-	-foreground		[styleConfig TEntry -foreground] \
-	-foreground,background	[styleConfig TEntry -foreground] \
-	-disabledforeground	[lindex [style map TEntry -foreground] 1] \
-	-selectbackground	[styleConfig . -selectbackground] \
-	-selectforeground	[styleConfig . -selectforeground] \
-	-selectborderwidth	[styleConfig . -selectborderwidth] \
-    ]
-}
-
 #
 # Private procedures related to global KDE configuration options
 # ==============================================================
@@ -733,7 +560,7 @@ proc mentry::getKdeConfigVal {group key} {
     variable kdeDirList
 
     if {![info exists kdeDirList]} {
-	makeKdeDirList 
+	makeKdeDirList
     }
 
     #
