@@ -9,7 +9,7 @@
 #   - Private procedures implementing the scrollableframe widget command
 #   - Private procedures used in bindings
 #
-# Copyright (c) 2019-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2019-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -26,7 +26,7 @@ namespace eval scrollutil::sf {
     # the widget to which the option applies: f stands for the outer frame and
     # w for the scrollableframe widget itself.
     #
-    #	Command-Line Name	 {Database Name		  Database Class     W}
+    #	Command-Line Name	{Database Name		Database Class       W}
     #	-----------------------------------------------------------------------
     #
     variable configSpecs
@@ -79,22 +79,22 @@ namespace eval scrollutil::sf {
 	    destroy $helpFrm
 	}
 
-	lappend configSpecs(-borderwidth) 0
-	lappend configSpecs(-cursor) ""
-	lappend configSpecs(-contentheight) 0
-	lappend configSpecs(-contentwidth) 0
-	lappend configSpecs(-fitcontentheight) 0
-	lappend configSpecs(-fitcontentwidth) 0
-	lappend configSpecs(-height) 100
-	lappend configSpecs(-relief) flat
-	lappend configSpecs(-takefocus) 0
-	lappend configSpecs(-width) 100
-	lappend configSpecs(-xscrollcommand) ""
-	lappend configSpecs(-xscrollincrement) 0
-	lappend configSpecs(-yscrollcommand) ""
-	lappend configSpecs(-yscrollincrement) 0
+	lappend configSpecs(-borderwidth)	0
+	lappend configSpecs(-cursor)		""
+	lappend configSpecs(-contentheight)	0
+	lappend configSpecs(-contentwidth)	0
+	lappend configSpecs(-fitcontentheight)	0
+	lappend configSpecs(-fitcontentwidth)	0
+	lappend configSpecs(-height)		7c
+	lappend configSpecs(-relief)		flat
+	lappend configSpecs(-takefocus)		0
+	lappend configSpecs(-width)		10c
+	lappend configSpecs(-xscrollcommand)	""
+	lappend configSpecs(-xscrollincrement)	0
+	lappend configSpecs(-yscrollcommand)	""
+	lappend configSpecs(-yscrollincrement)	0
     }
-    extendConfigSpecs 
+    extendConfigSpecs
 
     variable configOpts [lsort [array names configSpecs]]
 
@@ -102,10 +102,12 @@ namespace eval scrollutil::sf {
     # Use lists to facilitate the handling
     # of various options and corner values
     #
-    variable cmdOpts  [list cget configure contentframe scan see seerect \
-		       xview yview]
-    variable scanOpts [list mark dragto]
-    variable corners  [list nw ne sw se]
+    variable cmdOpts	[list attrib autofillx autofilly autosize cget \
+			 configure contentframe hasattrib scan see seerect \
+			 unsetattrib xview yview]
+    variable scanOpts	[list mark dragto]
+    variable dimensions	[list w h wh]
+    variable corners	[list nw ne sw se]
 
     #
     # Variables used in scan-related binding scripts:
@@ -155,6 +157,10 @@ proc scrollutil::sf::createBindings {} {
 	bind $class <ButtonRelease-1> \
 	    [list scrollutil::sf::onButtonRelease1 %W $isCf]
     }
+
+    bind ScrollableframeCf <<NoManagedChild>> {
+	scrollutil::sf::onNoManagedChild %W
+    }
 }
 
 #
@@ -205,17 +211,27 @@ proc scrollutil::scrollableframe args {
 	#
 	variable data
 	array set data {
+	    autoFillX	0
+	    autoFillY	0
 	    xOffset	0
 	    cfWidth	0
+	    cfReqWidth	0
 	    mfWidth	0
 	    yOffset	0
 	    cfHeight	0
+	    cfReqHeight	0
 	    mfHeight	0
 	    scanX	0
 	    scanY	0
 	    scanXOffset	0
 	    scanYOffset	0
 	}
+
+	#
+	# The following array is used to hold
+	# arbitrary attributes for this widget
+	#
+	variable attribs
     }
 
     #
@@ -232,12 +248,13 @@ proc scrollutil::scrollableframe args {
     # Create the middle frame of the class ScrollableframeMf
     # and the content frame of the class ScrollableframeCf
     #
-    foreach f [list $data(mf) $data(cf)] sfx [list Mf Cf] {
+    foreach f [list $data(mf) $data(cf)] suffix [list Mf Cf] {
 	if {$usingTile} {
-	    ttk::frame $f -class Scrollableframe$sfx -borderwidth 0 -height 0 \
-			  -padding 0 -relief flat -takefocus 0 -width 0
+	    ttk::frame $f -class Scrollableframe$suffix -borderwidth 0 \
+			  -height 0 -padding 0 -relief flat -takefocus 0 \
+			  -width 0
 	} else {
-	    tk::frame $f -class Scrollableframe$sfx -borderwidth 0 \
+	    tk::frame $f -class Scrollableframe$suffix -borderwidth 0 \
 			 -container 0 -height 0 -highlightthickness 0 \
 			 -relief flat -takefocus 0 -width 0
 	    catch {$f configure -padx 0 -pady 0}
@@ -429,7 +446,19 @@ proc scrollutil::sf::scrollableframeWidgetCmd {win args} {
 
     variable cmdOpts
     set cmd [mwutil::fullOpt "option" [lindex $args 0] $cmdOpts]
+    set argList [lrange $args 1 end]
+
     switch $cmd {
+	attrib {
+	    return [::scrollutil::attribSubCmd $win "widget" $argList]
+	}
+
+	autofillx { return [autofillxSubCmd $win $argList] }
+
+	autofilly { return [autofillySubCmd $win $argList] }
+
+	autosize  { return [autosizeSubCmd  $win $argList] }
+
 	cget {
 	    if {$argCount != 2} {
 		mwutil::wrongNumArgs "$win $cmd option"
@@ -447,8 +476,7 @@ proc scrollutil::sf::scrollableframeWidgetCmd {win args} {
 	configure {
 	    variable configSpecs
 	    return [mwutil::configureSubCmd $win configSpecs \
-		    scrollutil::sf::doConfig scrollutil::sf::doCget \
-		    [lrange $args 1 end]]
+		    scrollutil::sf::doConfig scrollutil::sf::doCget $argList]
 	}
 
 	contentframe {
@@ -460,18 +488,134 @@ proc scrollutil::sf::scrollableframeWidgetCmd {win args} {
 	    return $data(cf)
 	}
 
-	scan	{ return [scanSubCmd    $win [lrange $args 1 end]] }
+	hasattrib -
+	unsetattrib {
+	    if {$argCount != 2} {
+		mwutil::wrongNumArgs "$win $cmd name"
+	    }
 
-	see	{ return [seeSubCmd     $win [lrange $args 1 end]] }
+	    return [::scrollutil::${cmd}SubCmd $win "widget" [lindex $args 1]]
+	}
 
-	seerect	{ return [seerectSubCmd $win [lrange $args 1 end]] }
+	scan	{ return [scanSubCmd    $win $argList] }
 
-	xview	{ return [xviewSubCmd   $win [lrange $args 1 end]] }
+	see	{ return [seeSubCmd     $win $argList] }
 
-	yview	{ return [yviewSubCmd   $win [lrange $args 1 end]] }
+	seerect	{ return [seerectSubCmd $win $argList] }
+
+	xview	{ return [xviewSubCmd   $win $argList] }
+
+	yview	{ return [yviewSubCmd   $win $argList] }
     }
 }
 
+#------------------------------------------------------------------------------
+# scrollutil::sf::autofillxSubCmd
+#
+# Processes the scrollableframe autofillx subcommmand.
+#------------------------------------------------------------------------------
+proc scrollutil::sf::autofillxSubCmd {win argList} {
+    set argCount [llength $argList]
+    if {$argCount > 1} {
+	mwutil::wrongNumArgs "$win autofillx ?boolean?"
+    }
+
+    upvar ::scrollutil::ns${win}::data data
+    if {$argCount == 0} {
+	return $data(autoFillX)
+    } else {
+	set flag [expr {[lindex $argList 0] ? 1 : 0}]
+	set data(autoFillX) $flag
+
+	if {$flag} {
+	    set cfReqWidth [winfo reqwidth $data(cf)]
+	    set mfWidth [winfo width $data(mf)]
+	    set data(cfReqWidth) $cfReqWidth
+	    set data(mfWidth) $mfWidth
+	    doConfig $win -fitcontentwidth [expr {$cfReqWidth < $mfWidth}]
+	}
+
+	return ""
+    }
+}
+
+#------------------------------------------------------------------------------
+# scrollutil::sf::autofillySubCmd
+#
+# Processes the scrollableframe autofilly subcommmand.
+#------------------------------------------------------------------------------
+proc scrollutil::sf::autofillySubCmd {win argList} {
+    set argCount [llength $argList]
+    if {$argCount > 1} {
+	mwutil::wrongNumArgs "$win autofilly ?boolean?"
+    }
+
+    upvar ::scrollutil::ns${win}::data data
+    if {$argCount == 0} {
+	return $data(autoFillY)
+    } else {
+	set flag [expr {[lindex $argList 0] ? 1 : 0}]
+	set data(autoFillY) $flag
+	if {$flag} {
+	    set cfReqHeight [winfo reqheight $data(cf)]
+	    set mfHeight [winfo height $data(mf)]
+	    set data(cfReqHeight) $cfReqHeight
+	    set data(mfHeight) $mfHeight
+	    doConfig $win -fitcontentheight [expr {$cfReqHeight < $mfHeight}]
+	}
+
+	return ""
+    }
+}
+
+#------------------------------------------------------------------------------
+# scrollutil::sf::autosizeSubCmd
+#
+# Processes the scrollableframe autosize subcommmand.
+#------------------------------------------------------------------------------
+proc scrollutil::sf::autosizeSubCmd {win argList} {
+    set argCount [llength $argList]
+    if {$argCount > 1} {
+	mwutil::wrongNumArgs "$win autosize ?w|h|wh?"
+    }
+
+    #
+    # Parse the optional argument
+    #
+    set dimBits(width)  0
+    set dimBits(height) 0
+    if {$argCount == 1} {
+	variable dimensions
+	switch [mwutil::fullOpt "dimensions" [lindex $argList 0] $dimensions] {
+	    w  { set dimBits(width)  1 }
+	    h  { set dimBits(height) 1 }
+	    wh { set dimBits(width)  1; set dimBits(height) 1 }
+	}
+    }
+
+    upvar ::scrollutil::ns${win}::data data
+    if {$data(-fitcontentwidth)} {
+	set dimBits(width)  1
+    }
+    if {$data(-fitcontentheight)} {
+	set dimBits(height) 1
+    }
+
+    if {$dimBits(width) || $dimBits(height)} {
+	set dimList {}
+	if {$dimBits(width)} {
+	    lappend dimList width
+	}
+	if {$dimBits(height)} {
+	    lappend dimList height
+	}
+
+	after 100 [list scrollutil::sf::doAutosize $win $dimList]
+    }
+
+    return ""
+}
+
 #------------------------------------------------------------------------------
 # scrollutil::sf::scanSubCmd
 #
@@ -523,7 +667,7 @@ proc scrollutil::sf::scanSubCmd {win argList} {
 proc scrollutil::sf::seeSubCmd {win argList} {
     set argCount [llength $argList]
     if {$argCount < 1 || $argCount > 2} {
-	mwutil::wrongNumArgs "$win see widget ?nw|ne|sw|se?
+	mwutil::wrongNumArgs "$win see widget ?nw|ne|sw|se?"
     }
 
     set w [lindex $argList 0]
@@ -572,7 +716,7 @@ proc scrollutil::sf::seeSubCmd {win argList} {
 proc scrollutil::sf::seerectSubCmd {win argList} {
     set argCount [llength $argList]
     if {$argCount < 4 || $argCount > 5} {
-	mwutil::wrongNumArgs "$win seerect x1 y1 x2 y2 ?nw|ne|sw|se?
+	mwutil::wrongNumArgs "$win seerect x1 y1 x2 y2 ?nw|ne|sw|se?"
     }
 
     set x1 [format "%d" [lindex $argList 0]]
@@ -618,7 +762,7 @@ proc scrollutil::sf::seerectSubCmd {win argList} {
     set mfY2 [expr {$mfY1 + $mfHeight}]
 
     #
-    # Make the left or right part of w visible in the window
+    # Make the left or right edge of the rectangle visible in the window
     #
     switch $xSide {
 	w {
@@ -646,7 +790,7 @@ proc scrollutil::sf::seerectSubCmd {win argList} {
     applyOffset $win x $mfX1 0
 
     #
-    # Make the top or bottom part of w visible in the window
+    # Make the top or bottom edge of the rectangle visible in the window
     #
     switch $ySide {
 	n {
@@ -697,12 +841,27 @@ proc scrollutil::sf::xviewSubCmd {win argList} {
 	    }
 	    set first [expr {double($xOffset) / $cfWidth}]
 	    set last  [expr {double($xOffset + $mfWidth) / $cfWidth}]
-	    if {$last > 1.0} {
+	    if {$last == 0.0 || $last > 1.0} {
 		set last 1.0
 	    }
 	    return [list $first $last]
 	}
 
+	1 {
+	    #
+	    # Command: $win xview <units>
+	    #
+	    set units [format "%d" [lindex $argList 0]]
+	    set xScrlIncr $data(-xscrollincrement)
+	    if {$xScrlIncr > 0} {
+		set xOffset [expr {$units * $xScrlIncr}]
+	    } else {
+		set xOffset [expr {int($units * 0.1 * $mfWidth)}]
+	    }
+	    applyOffset $win x $xOffset 0
+	    return ""
+	}
+
 	default {
 	    #
 	    # Command: $win xview moveto <fraction>
@@ -755,12 +914,27 @@ proc scrollutil::sf::yviewSubCmd {win argList} {
 	    }
 	    set first [expr {double($yOffset) / $cfHeight}]
 	    set last  [expr {double($yOffset + $mfHeight) / $cfHeight}]
-	    if {$last > 1.0} {
+	    if {$last == 0.0 || $last > 1.0} {
 		set last 1.0
 	    }
 	    return [list $first $last]
 	}
 
+	1 {
+	    #
+	    # Command: $win yview <units>
+	    #
+	    set units [format "%d" [lindex $argList 0]]
+	    set yScrlIncr $data(-yscrollincrement)
+	    if {$yScrlIncr > 0} {
+		set yOffset [expr {$units * $yScrlIncr}]
+	    } else {
+		set yOffset [expr {int($units * 0.1 * $mfWidth)}]
+	    }
+	    applyOffset $win y $yOffset 0
+	    return ""
+	}
+
 	default {
 	    #
 	    # Command: $win yview moveto <fraction>
@@ -792,6 +966,24 @@ proc scrollutil::sf::yviewSubCmd {win argList} {
     }
 }
 
+#------------------------------------------------------------------------------
+# scrollutil::sf::doAutosize
+#------------------------------------------------------------------------------
+proc scrollutil::sf::doAutosize {win dimList} {
+    #
+    # This is an "after 100" callback; check whether the window exists
+    #
+    if {![array exists ::scrollutil::ns${win}::data] ||
+	[winfo class $win] ne "Scrollableframe"} {
+	return ""
+    }
+
+    upvar ::scrollutil::ns${win}::data data
+    foreach dim $dimList {
+	doConfig $win -$dim [winfo req$dim $data(cf)]
+    }
+}
+
 #------------------------------------------------------------------------------
 # scrollutil::sf::roundUp
 #------------------------------------------------------------------------------
@@ -880,21 +1072,34 @@ proc scrollutil::sf::applyOffset {win axis offset force} {
 #------------------------------------------------------------------------------
 # scrollutil::sf::onScrollableframeMfConfigure
 #------------------------------------------------------------------------------
-proc scrollutil::sf::onScrollableframeMfConfigure {mf width height} {
+proc scrollutil::sf::onScrollableframeMfConfigure {mf mfWidth mfHeight} {
     set win [winfo parent $mf]
     upvar ::scrollutil::ns${win}::data data
 
-    if {$width != $data(mfWidth)} {
-	set data(mfWidth) $width
+    if {$mfWidth != $data(mfWidth)} {
+	set data(mfWidth) $mfWidth
+
+	if {$data(autoFillX)} {
+	    doConfig $win -fitcontentwidth \
+		[expr {[winfo reqwidth $data(cf)] < $mfWidth}]
+	}
+
 	if {$data(-fitcontentwidth)} {
-	    set data(cfWidth) $width
+	    set data(cfWidth) $mfWidth
 	}
 	xviewSubCmd $win {scroll 0 units}
     }
-    if {$height != $data(mfHeight)} {
-	set data(mfHeight) $height
+
+    if {$mfHeight != $data(mfHeight)} {
+	set data(mfHeight) $mfHeight
+
+	if {$data(autoFillY)} {
+	    doConfig $win -fitcontentheight \
+		[expr {[winfo reqheight $data(cf)] < $mfHeight}]
+	}
+
 	if {$data(-fitcontentheight)} {
-	    set data(cfHeight) $height
+	    set data(cfHeight) $mfHeight
 	}
 	yviewSubCmd $win {scroll 0 units}
     }
@@ -903,16 +1108,29 @@ proc scrollutil::sf::onScrollableframeMfConfigure {mf width height} {
 #------------------------------------------------------------------------------
 # scrollutil::sf::onScrollableframeCfConfigure
 #------------------------------------------------------------------------------
-proc scrollutil::sf::onScrollableframeCfConfigure {cf width height} {
+proc scrollutil::sf::onScrollableframeCfConfigure {cf cfWidth cfHeight} {
     set win [winfo parent [winfo parent $cf]]
     upvar ::scrollutil::ns${win}::data data
 
-    if {$width != $data(cfWidth)} {
-	set data(cfWidth) $width
+    if {$data(autoFillX) &&
+	[set cfReqWidth [winfo reqwidth $data(cf)]] != $data(cfReqWidth)} {
+	set data(cfReqWidth) $cfReqWidth
+	doConfig $win -fitcontentwidth \
+	    [expr {$cfReqWidth < [winfo width $data(mf)]}]
+    }
+    if {$data(autoFillY) &&
+	[set cfReqHeight [winfo reqheight $data(cf)]] != $data(cfReqHeight)} {
+	set data(cfReqHeight) $cfReqHeight
+	doConfig $win -fitcontentheight \
+	    [expr {$cfReqHeight < [winfo height $data(mf)]}]
+    }
+
+    if {$cfWidth != $data(cfWidth)} {
+	set data(cfWidth) $cfWidth
 	xviewSubCmd $win {scroll 0 units}
     }
-    if {$height != $data(cfHeight)} {
-	set data(cfHeight) $height
+    if {$cfHeight != $data(cfHeight)} {
+	set data(cfHeight) $cfHeight
 	yviewSubCmd $win {scroll 0 units}
     }
 }
@@ -978,3 +1196,21 @@ proc scrollutil::sf::onButtonRelease1 {w isCf} {
     variable origCursor
     ::$win configure -cursor $origCursor
 }
+
+#------------------------------------------------------------------------------
+# scrollutil::sf::onNoManagedChild
+#------------------------------------------------------------------------------
+proc scrollutil::sf::onNoManagedChild cf {
+    set win [winfo parent [winfo parent $cf]]
+    upvar ::scrollutil::ns${win}::data data
+
+    if {!$data(-fitcontentwidth) && $data(-contentwidth) <= 0} {
+	$cf configure -width 1
+	$cf configure -width $data(-contentwidth)
+    }
+
+    if {!$data(-fitcontentheight) && $data(-contentheight) <= 0} {
+	$cf configure -height 1
+	$cf configure -height $data(-contentheight)
+    }
+}
