@@ -11,7 +11,7 @@
 #   - Private procedures used in bindings
 #   - Private utility procedures
 #
-# Copyright (c) 2019-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2019-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -70,7 +70,7 @@ namespace eval scrollutil::sa {
     # the widget to which the option applies: f stands for the frame and w for
     # the scrollarea widget itself.
     #
-    #	Command-Line Name	 {Database Name		  Database Class     W}
+    #	Command-Line Name	{Database Name		Database Class       W}
     #	-----------------------------------------------------------------------
     #
     variable configSpecs
@@ -88,6 +88,7 @@ namespace eval scrollutil::sa {
 	-relief			{relief			Relief		     f}
 	-respectheader		{respectHeader		RespectHeader	     w}
 	-respecttitlecolumns	{respectTitleColumns	RespectTitleColumns  w}
+	-setfocus		{setFocus		SetFocus	     w}
 	-takefocus		{takeFocus		TakeFocus	     f}
 	-xscrollbarmode		{xScrollbarMode		ScrollbarMode	     w}
 	-yscrollbarmode		{yScrollbarMode		ScrollbarMode	     w}
@@ -120,19 +121,20 @@ namespace eval scrollutil::sa {
 	    destroy $helpFrm
 	}
 
-	lappend configSpecs(-autohidescrollbars) 0
-	lappend configSpecs(-borderwidth) 1
-	lappend configSpecs(-cursor) ""
-	lappend configSpecs(-lockinterval) 1
-	lappend configSpecs(-relief) sunken
+	lappend configSpecs(-autohidescrollbars)	0
+	lappend configSpecs(-borderwidth)		1
+	lappend configSpecs(-cursor)			""
+	lappend configSpecs(-lockinterval)		1
+	lappend configSpecs(-relief)			sunken
 	lappend configSpecs(-respectheader) \
 		[expr {[string compare $winSys "win32"] != 0}]
-	lappend configSpecs(-respecttitlecolumns) 1
-	lappend configSpecs(-takefocus) 0
-	lappend configSpecs(-xscrollbarmode) dynamic
-	lappend configSpecs(-yscrollbarmode) dynamic
+	lappend configSpecs(-respecttitlecolumns)	1
+	lappend configSpecs(-setfocus)			0
+	lappend configSpecs(-takefocus)			0
+	lappend configSpecs(-xscrollbarmode)		dynamic
+	lappend configSpecs(-yscrollbarmode)		dynamic
     }
-    extendConfigSpecs 
+    extendConfigSpecs
 
     variable configOpts [lsort [array names configSpecs]]
 
@@ -140,8 +142,15 @@ namespace eval scrollutil::sa {
     # Use lists to facilitate the handling of
     # the command options and scrollbar modes
     #
-    variable cmdOpts        [list cget configure setwidget widget]
+    variable cmdOpts        [list attrib cget configure hasattrib setwidget \
+			     unsetattrib widget]
     variable scrollbarModes [list static dynamic none]
+
+    variable newSetFocusPolicy [expr {
+	($::tk_version == 8.6 &&
+	 [package vcompare $::tk_patchLevel "8.6.11"] >= 0) ||
+	($::tk_version >= 8.7 &&
+	 [package vcompare $::tk_patchLevel "8.7a4"] >= 0)}]
 }
 
 #
@@ -153,7 +162,7 @@ namespace eval scrollutil::sa {
 # scrollutil::sa::createBindings
 #
 # Creates the default bindings for the binding tags Scrollarea, ScrollareaTop,
-# DynamicHScrollbar, and WidgetOfScrollarea.
+# ScrollareaScrollbar, DynamicHScrollbar, and WidgetOfScrollarea.
 #------------------------------------------------------------------------------
 proc scrollutil::sa::createBindings {} {
     bind Scrollarea <KeyPress> continue
@@ -163,10 +172,9 @@ proc scrollutil::sa::createBindings {} {
         }
     }
     bind Scrollarea <Configure> {
+	after   1 [list scrollutil::sa::updateScrollbars %W]
 	after 300 [list scrollutil::sa::updateScrollbars %W]
     }
-    bind Scrollarea <Map>        { set scrollutil::ns%W::data(isMapped) 1 }
-    bind Scrollarea <Unmap>      { set scrollutil::ns%W::data(isMapped) 0 }
     bind Scrollarea <Enter>	 { scrollutil::sa::onScrollareaEnter %W }
     bind Scrollarea <Leave>	 { scrollutil::sa::onScrollareaLeave %W }
     bind Scrollarea <Destroy> {
@@ -181,6 +189,13 @@ proc scrollutil::sa::createBindings {} {
 	after 100 [list scrollutil::sa::onToplevelFocusOut %W]
     }
 
+    bind ScrollareaScrollbar <Button-1> {
+	scrollutil::sa::onScrollbarClicked %W
+    }
+    bind ScrollareaScrollbar <Button-2> {
+	scrollutil::sa::onScrollbarClicked %W
+    }
+
     bind DynamicHScrollbar <Map> { scrollutil::sa::onDynamicHScrollbarMap %W }
 
     bind WidgetOfScrollarea <Destroy> {
@@ -235,7 +250,6 @@ proc scrollutil::scrollarea args {
 	#
 	variable data
 	array set data {
-	    isMapped	 0
 	    hsbManaged	 0
 	    vsbManaged	 0
 	    hsbLocked	 0
@@ -247,6 +261,12 @@ proc scrollutil::scrollarea args {
 	    cf-ne_height 1
 	    cf-sw_width  1
 	}
+
+	#
+	# The following array is used to hold
+	# arbitrary attributes for this widget
+	#
+	variable attribs
     }
 
     #
@@ -284,6 +304,7 @@ proc scrollutil::scrollarea args {
 	$sb configure -takefocus 0
 	$sb.f configure -borderwidth 0 -height 0 -relief flat -takefocus 0 \
 			-width 0
+	bindtags $sb [linsert [bindtags $sb] 1 ScrollareaScrollbar]
     }
 
     #
@@ -392,7 +413,7 @@ proc scrollutil::sa::doConfig {win opt val} {
 
 		    if {$data($opt)} {
 			if {[mwutil::hasFocus [winfo toplevel $win]] &&
-			    $data(isMapped) && [containsPointer $win]} {
+			    [mwutil::containsPointer $win]} {
 			    unobscureScrollbars $win
 			} else {
 			    obscureScrollbars $win
@@ -422,15 +443,29 @@ proc scrollutil::sa::doConfig {win opt val} {
 			showHScrollbar $win 1
 		    }
 		}
+		-setfocus {
+		    set data($opt) [expr {$val ? 1 : 0}]
+		}
 		-xscrollbarmode {
 		    variable scrollbarModes
 		    set val \
 			[mwutil::fullOpt "scrollbar mode" $val $scrollbarModes]
-		    if {[string compare $val "none"] != 0 &&
-			[winfo exists $data(widget)]} {
-			if {![mwutil::isScrollable $data(widget) x]} {
-			    return -code error "widget $data(widget) fails to\
-				support horizontal scrolling"
+		    if {[winfo exists $data(widget)]} {
+			if {[string compare $val "none"] == 0} {
+			    ::$data(widget) configure -xscrollcommand ""
+			    $win.hsb configure -command ""
+			} else {
+			    if {![mwutil::isScrollable $data(widget) x]} {
+				return -code error "widget $data(widget) fails\
+				    to support horizontal scrolling"
+			    }
+
+			    if {[string compare $val $data($opt)] != 0} {
+				::$data(widget) configure -xscrollcommand \
+				    [list scrollutil::sa::setHScrollbar $win]
+				$win.hsb configure -command \
+				    [list $data(widget) xview]
+			    }
 			}
 		    }
 		    set data($opt) $val
@@ -458,11 +493,22 @@ proc scrollutil::sa::doConfig {win opt val} {
 		    variable scrollbarModes
 		    set val \
 			[mwutil::fullOpt "scrollbar mode" $val $scrollbarModes]
-		    if {[string compare $val "none"] != 0 &&
-			[winfo exists $data(widget)]} {
-			if {![mwutil::isScrollable $data(widget) y]} {
-			    return -code error "widget $data(widget) fails to\
-				support vertical scrolling"
+		    if {[winfo exists $data(widget)]} {
+			if {[string compare $val "none"] == 0} {
+			    ::$data(widget) configure -yscrollcommand ""
+			    $win.vsb configure -command ""
+			} else {
+			    if {![mwutil::isScrollable $data(widget) y]} {
+				return -code error "widget $data(widget) fails\
+				    to support vertical scrolling"
+			    }
+
+			    if {[string compare $val $data($opt)] != 0} {
+				::$data(widget) configure -yscrollcommand \
+				    [list scrollutil::sa::setVScrollbar $win]
+				$win.vsb configure -command \
+				    [list $data(widget) yview]
+			    }
 			}
 		    }
 		    set data($opt) $val
@@ -509,7 +555,13 @@ proc scrollutil::sa::scrollareaWidgetCmd {win args} {
 
     variable cmdOpts
     set cmd [mwutil::fullOpt "option" [lindex $args 0] $cmdOpts]
+
     switch $cmd {
+	attrib {
+	    return [::scrollutil::attribSubCmd $win "widget" \
+		    [lrange $args 1 end]]
+	}
+
 	cget {
 	    if {$argCount != 2} {
 		mwutil::wrongNumArgs "$win $cmd option"
@@ -531,6 +583,15 @@ proc scrollutil::sa::scrollareaWidgetCmd {win args} {
 		    [lrange $args 1 end]]
 	}
 
+	hasattrib -
+	unsetattrib {
+	    if {$argCount != 2} {
+		mwutil::wrongNumArgs "$win $cmd name"
+	    }
+
+	    return [::scrollutil::${cmd}SubCmd $win "widget" [lindex $args 1]]
+	}
+
 	setwidget {
 	    if {$argCount != 2} {
 		mwutil::wrongNumArgs "$win $cmd widget"
@@ -687,7 +748,7 @@ proc scrollutil::sa::setHScrollbar {win first last} {
     upvar ::scrollutil::ns${win}::data data
     $win.hsb set $first $last
 
-    if {$data(sbObscured) && $data(isMapped) && [containsPointer $win]} {
+    if {$data(sbObscured) && [mwutil::containsPointer $win]} {
 	unobscureScrollbars $win
     }
 
@@ -712,7 +773,7 @@ proc scrollutil::sa::setVScrollbar {win first last} {
     upvar ::scrollutil::ns${win}::data data
     $win.vsb set $first $last
 
-    if {$data(sbObscured) && $data(isMapped) && [containsPointer $win]} {
+    if {$data(sbObscured) && [mwutil::containsPointer $win]} {
 	unobscureScrollbars $win
     }
 
@@ -778,7 +839,7 @@ proc scrollutil::sa::onToplevelFocusIn top {
 	set lst2 {}
 	foreach w $lst1 {
 	    if {[string compare [winfo class $w] "Scrollarea"] == 0 &&
-		[winfo ismapped $w] && [containsPointer $w]} {
+		[mwutil::containsPointer $w]} {
 		unobscureScrollbars $w
 	    }
 	    foreach child [winfo children $w] {
@@ -816,6 +877,51 @@ proc scrollutil::sa::onToplevelFocusOut top {
     }
 }
 
+#------------------------------------------------------------------------------
+# scrollutil::sa::onScrollbarClicked
+#------------------------------------------------------------------------------
+proc scrollutil::sa::onScrollbarClicked sb {
+    set win [winfo parent $sb]
+    set widget [::$win widget]
+    if {![::$win cget -setfocus] || ![winfo exists $widget]} {
+	return ""
+    }
+
+    switch [winfo class $widget] {
+	Text {
+	    variable ::scrollutil::winSys
+	    variable newSetFocusPolicy
+	    if {$winSys eq "win32" || $newSetFocusPolicy} {
+		focus $widget
+	    }
+	}
+
+	Ctext {
+	    variable ::scrollutil::winSys
+	    variable newSetFocusPolicy
+	    if {$winSys eq "win32" || $newSetFocusPolicy} {
+		focus $widget.t
+	    }
+	}
+
+	Treeview { focus $widget }
+
+	default {
+	    if {[catch {$widget instate disabled} result] == 0} {
+		set isDisabled $result
+	    } elseif {[catch {$widget cget -state} result] == 0} {
+		set isDisabled [expr {[string compare $result "disabled"] == 0}]
+	    } else {
+		set isDisabled 0
+	    }
+
+	    if {!$isDisabled} {
+		focus $widget
+	    }
+	}
+    }
+}
+
 #------------------------------------------------------------------------------
 # scrollutil::sa::onDynamicHScrollbarMap
 #------------------------------------------------------------------------------
@@ -847,14 +953,15 @@ proc scrollutil::sa::onDynamicHScrollbarMap hsb {
 # scrollutil::sa::onWidgetOfScrollareaDestroy
 #------------------------------------------------------------------------------
 proc scrollutil::sa::onWidgetOfScrollareaDestroy widget {
-    variable scrollareaArr
-    if {[info exists scrollareaArr($widget)]} {
-	unset scrollareaArr($widget)
-    }
-
     set win [::scrollutil::getscrollarea $widget]
     if {[string length $win] != 0} {
 	::$win setwidget ""
+	$win configure -width 1 -height 1
+    }
+
+    variable scrollareaArr
+    if {[info exists scrollareaArr($widget)]} {
+	unset scrollareaArr($widget)
     }
 }
 
@@ -902,7 +1009,9 @@ proc scrollutil::sa::onTitleColsWidthChanged tbl {
 #------------------------------------------------------------------------------
 proc scrollutil::sa::showHScrollbar {win {redisplay 0}} {
     upvar ::scrollutil::ns${win}::data data
-    if {$data(vsbLocked) || ($data(hsbManaged) && !$redisplay)} {
+    if {($data(vsbLocked) &&
+	 [string compare $data(-xscrollbarmode) "dynamic"] == 0) ||
+	($data(hsbManaged) && !$redisplay)} {
 	return ""
     }
 
@@ -976,7 +1085,9 @@ proc scrollutil::sa::unlockHScrollbar win {
 #------------------------------------------------------------------------------
 proc scrollutil::sa::showVScrollbar {win {redisplay 0}} {
     upvar ::scrollutil::ns${win}::data data
-    if {$data(hsbLocked) || ($data(vsbManaged) && !$redisplay)} {
+    if {($data(hsbLocked) &&
+	 [string compare $data(-yscrollbarmode) "dynamic"] == 0) ||
+	($data(vsbManaged) && !$redisplay)} {
 	return ""
     }
 
@@ -1057,7 +1168,6 @@ proc scrollutil::sa::obscureScrollbars win {
     set data(sbObscured) 1
     place $win.hsb.f -bordermode outside -relheight 1.0 -relwidth 1.0
     place $win.vsb.f -bordermode outside -relheight 1.0 -relwidth 1.0
-    
 }
 
 #------------------------------------------------------------------------------
@@ -1074,19 +1184,6 @@ proc scrollutil::sa::unobscureScrollbars win {
     place forget $win.vsb.f
 }
 
-#------------------------------------------------------------------------------
-# scrollutil::sa::containsPointer
-#------------------------------------------------------------------------------
-proc scrollutil::sa::containsPointer win {
-    foreach {ptrX ptrY} [winfo pointerxy $win] {}
-    set winX [winfo rootx $win]
-    set winY [winfo rooty $win]
-    return [expr {
-	$ptrX >= $winX && $ptrX < $winX + [winfo width  $win] &&
-	$ptrY >= $winY && $ptrY < $winY + [winfo height $win]
-    }]
-}
-
 #------------------------------------------------------------------------------
 # scrollutil::sa::containsTextWidget
 #------------------------------------------------------------------------------
