@@ -11,7 +11,7 @@
 #   - Private procedures used in bindings
 #   - Private utility procedures
 #
-# Copyright (c) 2019-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2019-2022  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
 #
@@ -28,7 +28,7 @@ namespace eval scrollutil::ss {
     # the widget to which the option applies: f stands for the frame and w for
     # the scrollsync widget itself.
     #
-    #	Command-Line Name	 {Database Name		  Database Class     W}
+    #	Command-Line Name	{Database Name		Database Class       W}
     #	-----------------------------------------------------------------------
     #
     variable configSpecs
@@ -73,21 +73,22 @@ namespace eval scrollutil::ss {
 	    destroy $helpFrm
 	}
 
-	lappend configSpecs(-borderwidth) 0
-	lappend configSpecs(-cursor) ""
-	lappend configSpecs(-relief) flat
-	lappend configSpecs(-takefocus) 0
-	lappend configSpecs(-xscrollcommand) ""
-	lappend configSpecs(-yscrollcommand) ""
+	lappend configSpecs(-borderwidth)	0
+	lappend configSpecs(-cursor)		""
+	lappend configSpecs(-relief)		flat
+	lappend configSpecs(-takefocus)		0
+	lappend configSpecs(-xscrollcommand)	""
+	lappend configSpecs(-yscrollcommand)	""
     }
-    extendConfigSpecs 
+    extendConfigSpecs
 
     variable configOpts [lsort [array names configSpecs]]
 
     #
     # Use a list to facilitate the handling of the command options
     #
-    variable cmdOpts [list cget configure setwidgets widgets xview yview]
+    variable cmdOpts [list attrib cget configure hasattrib setwidgets \
+		      unsetattrib widgets xview yview]
 }
 
 #
@@ -175,6 +176,12 @@ proc scrollutil::scrollsync args {
 	    xScrollableList	{}
 	    yScrollableList	{}
 	}
+
+	#
+	# The following array is used to hold
+	# arbitrary attributes for this widget
+	#
+	variable attribs
     }
 
     #
@@ -298,7 +305,13 @@ proc scrollutil::ss::scrollsyncWidgetCmd {win args} {
 
     variable cmdOpts
     set cmd [mwutil::fullOpt "option" [lindex $args 0] $cmdOpts]
+
     switch $cmd {
+	attrib {
+	    return [::scrollutil::attribSubCmd $win "widget" \
+		    [lrange $args 1 end]]
+	}
+
 	cget {
 	    if {$argCount != 2} {
 		mwutil::wrongNumArgs "$win $cmd option"
@@ -320,6 +333,15 @@ proc scrollutil::ss::scrollsyncWidgetCmd {win args} {
 		    [lrange $args 1 end]]
 	}
 
+	hasattrib -
+	unsetattrib {
+	    if {$argCount != 2} {
+		mwutil::wrongNumArgs "$win $cmd name"
+	    }
+
+	    return [::scrollutil::${cmd}SubCmd $win "widget" [lindex $args 1]]
+	}
+
 	setwidgets {
 	    if {$argCount != 2} {
 		mwutil::wrongNumArgs "$win $cmd widgetList"
@@ -534,11 +556,6 @@ proc scrollutil::ss::updateMasterWidgets win {
 # scrollutil::ss::onWidgetOfScrollsyncDestroy
 #------------------------------------------------------------------------------
 proc scrollutil::ss::onWidgetOfScrollsyncDestroy widget {
-    variable scrollsyncArr
-    if {[info exists scrollsyncArr($widget)]} {
-	unset scrollsyncArr($widget)
-    }
-
     variable xViewArr
     if {[info exists xViewArr($widget)]} {
 	unset xViewArr($widget)
@@ -555,6 +572,11 @@ proc scrollutil::ss::onWidgetOfScrollsyncDestroy widget {
 	set idx [lsearch -exact $widgetList $widget]
 	::$win setwidgets [lreplace $widgetList $idx $idx]
     }
+
+    variable scrollsyncArr
+    if {[info exists scrollsyncArr($widget)]} {
+	unset scrollsyncArr($widget)
+    }
 }
 
 #
