@@ -1,11 +1,11 @@
 #==============================================================================
 # Main Tablelist package module.
 #
-# Copyright (c) 2000-2020  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2000-2024  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require Tk 8
-package require -exact tablelist::common 6.11
+package require Tk 8.4-
+package require -exact tablelist::common 7.1
 
 package provide tablelist $::tablelist::version
 package provide Tablelist $::tablelist::version
